@@ -150,18 +150,23 @@ class GroupBy:
                 for n in self._index.keys())
         )
 
-    def get_index_column_expressions(self, construct_multi_levels: bool = False) -> List[Expression]:
+    def get_index_column_expressions(self, construct_multi_levels: bool = False) -> Dict[str, Expression]:
+        """
+        returns a mapping between column name and expression
+        """
         if construct_multi_levels:
-            return [g.get_column_expression() for g in self._index.values()]
+            return {g.name: g.get_column_expression() for g in self._index.values()}
 
         from bach.series import SeriesAbstractMultiLevel
-        exprs = [
-            [g.get_column_expression()] if not isinstance(g, SeriesAbstractMultiLevel)
-            else g.get_all_level_column_expression()
-            for g in self._index.values()
-        ]
+        exprs = {}
+        for g in self._index.values():
+            if not isinstance(g, SeriesAbstractMultiLevel):
+                exprs[g.name] = g.get_column_expression()
+                continue
+            for lvl in g.levels.values():
+                exprs[lvl.name] = lvl.get_column_expression()
 
-        return list(itertools.chain.from_iterable(exprs))
+        return exprs
 
     def get_index_expressions(self) -> List[Expression]:
         from bach.series import SeriesAbstractMultiLevel
