@@ -32,6 +32,11 @@ class SeriesBoolean(Series, ABC):
 
     Boolean Series can be created from int and string values. Not all conversions errors will be caught on
     conversion time. Some will lead to database errors later.
+
+    **Database support and types**
+
+    * Postgres: utilizes the 'boolean' database type.
+    * BigQuery: utilizes the 'BOOL' database type.
     """
     dtype = 'bool'
     dtype_aliases = ('boolean', '?', bool)
@@ -104,7 +109,8 @@ class SeriesBoolean(Series, ABC):
 
     def min(self, partition: WrappedPartition = None, skipna: bool = True):
         """
-        Returns the minimum value in the partition.
+        Returns the minimum value in the partition, i.e. value will be True iff all values in the
+        partition are True.
 
         :param partition: The partition or window to apply
         :param skipna: only ``skipna=True`` supported. This means NULL values are ignored.
@@ -114,7 +120,8 @@ class SeriesBoolean(Series, ABC):
 
     def max(self, partition: WrappedPartition = None, skipna: bool = True):
         """
-        Returns the maximum value in the partition.
+        Returns the maximum value in the partition, i.e. value will be False iff all values in the
+        partition are False.
 
         :param partition: The partition or window to apply
         :param skipna: only ``skipna=True`` supported. This means NULL values are ignored.
